"""Contains a wrapper for the Pastebin API for easier usage."""

import requests
import os
import pbwrap.formatter as formatter
from pbwrap.constants import API_OPTIONS


class Pastebin(object):
    """Pastebin class represents your communication with the Pastebin API through its functions
       you can use every API endpoint avalaible.

       Most functions require at least an api_dev_key parameter.
       Functions for manipulating your pastes through the API require an api_user_key.
    """

    def __init__(self, api_dev_key=None):
        """Instantiate a Pastebin Object

           :param api_dev_key: Your API Pastebin key
           :type api_dev_key: string
        """
        self.api_dev_key = api_dev_key
        self.api_user_key = None

    def authenticate(self, username, password):
        """Authenticate through the API login endpoint
           Your api_user_key attribute is set automatically

           :type username: string
           :param username: Your username

           :type password: string
           :param password: Your password

           :returns: your user_id key
           :rtype: string
        """
        data = {'api_dev_key': self.api_dev_key,
                'api_user_name': username,
                'api_user_password': password}

        r = requests.post('https://pastebin.com/api/api_login.php', data)

        self.api_user_key = r.text

        return self.api_user_key

    def get_user_details(self):
        """Return user details in a dictionary.
           Can only be user after authenticating with get_user_id(username, password).

           :returns: dictionary containing user details
           :rtype: dictionary
        """
        data = {'api_dev_key': self.api_dev_key,
                'api_user_key': self.api_user_key}

        r = requests.post('https://pastebin.com/api/api_post.php', data)

        return formatter.user_from_xml(r.text)

    def get_trending(self):
        """Return a list of paste objects created from the most trending pastes

           :returns: a list of Paste objects
           :rtype: list
        """
        data = {
            'api_dev_key': self.api_dev_key,
            'api_option': API_OPTIONS['TREND']}

        r = requests.post('https://pastebin.com/api/api_post.php', data)

        return formatter.paste_list_from_xml(r.text)

    @staticmethod
    def get_archive():
        """Return archive paste link list.Archive contains 25 most recent pastes.

           :returns: a list of url strings
           :rtype: list
        """
        r = requests.get('https://pastebin.com/archive')

        return formatter.archive_url_format(r.text)

    def get_raw_paste(self, paste_id):
        """Return raw string of given paste_id.

           :type paste_id: string
           :param paste_id: The ID key of the paste

           :returns: the text of the paste
           :rtype: string
        """
        r = requests.get('https://pastebin.com/raw/' + paste_id)
        return r.text

    def create_paste(
            self,
            api_paste_code,
            api_paste_private=0,
            api_paste_name=None,
            api_paste_expire_date=None,
            api_paste_format=None):
        """Create a new paste if succesfull return it's url.

           :type api_paste_code: string
           :param api_paste_code: your paste text

           :type api_paste_private: int
           :param api_paste_private: valid values=0(public),1(unlisted),2(private)

           :type api_paste_name: string
           :param api_user_name: your paste name

           :type api_paste_expire_date: string
           :param api_paste_expire_date: check documentation for valid values

           :type api_paste_format: string
           :param api_paste_format: check documentation for valid values

           :returns: new paste url
           :rtype: string
        """
        data = {'api_dev_key': self.api_dev_key,
                'api_user_key': self.api_user_key,
                'api_paste_code': api_paste_code,
                'api_paste_private': api_paste_private,
                'api_paste_name': api_paste_name,
                'api_paste_expire_date': api_paste_expire_date,
                'api_paste_format': api_paste_format,
                'api_option': API_OPTIONS['PASTE']}

        # Filter data and remove dictionary None keys.
        filtered_data = {k: v for k, v in data.items() if v is not None}

        r = requests.post('https://pastebin.com/api/api_post.php', filtered_data)

        return r.text

    def create_paste_from_file(
            self,
            filepath,
            api_paste_private=0,
            api_paste_name=None,
            api_paste_expire_date=None,
            api_paste_format=None):
            """Create a new paste from file if succesfull return it's url.

                :type filepath: string
                :param filepath: the path of the file

                :type api_paste_private: int
                :param api_paste_private: valid values=0(public),1(unlisted),2(private)

                :type api_paste_name: string
                :param api_user_name: your paste name

                :type api_paste_expire_date: string
                :param api_paste_expire_date: check documentation for valid values

                :type api_paste_format: string
                :param api_paste_format: check documentation for valid values

                :returns: new paste url
                :rtype: string
            """
            if os.path.exists(filepath):
                api_paste_code = open(filepath).read()
                return create_paste(api_paste_code, api_paste_private,
                                    api_paste_name, api_paste_expire_date, api_paste_format)

            print('File not found')
            return None

    def get_user_pastes(self, api_results_limit=None):
        """Return a list of Pastes created from the user

            :type api_results_limit: int
            :param api_results_limit: min=1, max=1000

            :returns: a list of Pastes created from the user
            :rtype: list
        """
        data = {'api_dev_key': self.api_dev_key,
                'api_user_key': self.api_user_key,
                'api_results_limit': api_results_limit,
                'api_option': API_OPTIONS['USER_PASTE']}

        # Filter data and remove dictionary None keys.
        filtered_data = {k: v for k, v in data.items() if v is not None}

        r = requests.post('https://pastebin.com/api/api_post.php', filtered_data)

        if r.text:
            return formatter.paste_list_from_xml(r.text)

        return 'No pastes in this account'

    def get_user_raw_paste(self, api_paste_key):
        """Return the raw data of a user paste(even private pastes!) as string.

            :param api_paste_key: the id key of the paste you want to fetch
            :type api_paste_key: string

            :returns: the text of the paste
            :rtype: string
        """
        data = {'api_dev_key': self.api_dev_key,
                'api_user_key': self.api_user_key,
                'api_paste_key': api_paste_key,
                'api_option': API_OPTIONS['USER_RAW_PASTE']}

        r = requests.post('https://pastebin.com/api/api_post.php', data)

        return r.text

    def delete_user_paste(self, api_paste_key):
        """Deletes a paste created by the user.

            :param api_paste_key: the id key of the paste you want to delete
            :type api_paste_key: string

            :returns: api response
            :rtype: string
        """
        data = {'api_dev_key': self.api_dev_key,
                'api_user_key': self.api_user_key,
                'api_paste_key': api_paste_key,
                'api_option': API_OPTIONS['DELETE_PASTE']}

        r = requests.post('https://pastebin.com/api/api_post.php', data)

        return r.text



"""A helper module containing the Constants used in the wrapper."""
API_OPTIONS = {'TREND': 'trends',
               'PASTE': 'paste',
               'USER_PASTE': 'list',
               'DELETE_PASTE': 'delete',
               'USER_RAW_PASTE': 'show_paste'}



class Paste(object):
    """Defines a Paste from Pastebin paste contains the following fields:
       key,
       date,
       title,
       size,
       expire_date,
       private,
       format_short,
       format_long,
       url,
       hits.
    """

    def __init__(self, paste_dict):
        for k, v in paste_dict.items():
            setattr(self, k, v)

    def __cmp__(self, x):
        return vars(self) == vars(x)


class User(object):
    """Defines a user contains the following fields:
       name
       format_short
       expiration
       avatar_url
       private
       website
       email
       location
       account_type
    """
    def __init__(self, user_dict):
        for k, v in user_dict.items():
            setattr(self, k, v)



"""Contains function that manipulate the API returns for easier data manipulation."""
import xml.etree.ElementTree as et
import re
from pbwrap.models import Paste


def paste_list_from_xml(xml_paste):
    """Input an xml list and return a list of paste objects.

        :param xml_paste: An xml formatted response containing Paste information
        :type: string

        :returns: a list of Paste objects parsed from the input xml formatted string
        :rtype: list
    """
    paste_list = list()

    # xml demands a base root to create xml element from string.
    # So we have to hardcode it.
    root = et.fromstring("<root>" + xml_paste + "</root>")

    # Iterate <paste> child elements and create a paste object.
    for paste_root in root:
        paste_dict = dict()

        for paste_element in paste_root:
            key = paste_element.tag.split('_', 1)[-1]
            value = paste_element.text

            paste_dict[key] = value

        paste_list.append(Paste(paste_dict))

    return paste_list


def archive_url_format(archive_html):
    """Return a list with recent pastes urls

        :param archive_html: raw html of the archive url
        :type archive_html: string

        :returns: a list containing url strings
        :rtype: list
    """
    pastes_urls = list()
    # Regex Magic
    pastes = re.findall(r'/><a href=\"/(.+?)\">', archive_html)

    for paste_id in pastes:

        if re.match(r'[a-zA-Z0-9]{8}', paste_id) is not None:
            pastes_urls.append('https://pastebin.com/' + paste_id)

    print(pastes_urls)
    return pastes_urls


def user_from_xml(user_xml_string):
    """Return user dictionary from an xml format string

        :param user_xml_string: xml formatted string containing user information
        :type user_xml_string: string

        :returns: A dictionary containing the user info
        :rtype: dictionary
    """
    root = et.fromstring(user_xml_string)
    user_dict = dict()

    for user in root:
        key = user.tag.split('user_')[-1]
        value = user.text
        user_dict[key] = value

    return user_dict
